(* ::Package:: *)

(*:Package Version: 2.06 *)

(*:Mathematica Version: 8.01 *)

(*:Version History: 1.01 (1999.07.15 - 2011.06.03)
                    2.01 (2011.06.04 - 2011.08.24)
                    2.02 (2011.08.25 - 2012.11.22)
                    2.03 (2012.11.23 - 2015.04.01)
                    2.04 (2015.04.02 - 2018.01.27)Clausen
                    2.05 (2018.01.28 - 2018.05.13)polylog rel. -> Zeta.m
                    2.06 (2018.05.14 -     .  .  )GoodwinStaton -> KummerHyp.m *)

(*:Copyright: Copyright 1999.07-2018.05, Souichiro Ikebe *)

(*:Name: SpecialFunctions`Miscellaneous` *)

(*:Title: Miscellaneous *)

(*:Keywords: SievertIntegral, SievertIntegralPrime, 
    SinSievertIntegral, SinSievertIntegralPrime, 
    CosSievertIntegral, CosSievertIntegralPrime, 
    AbramowitzF, AbramowitzFExp, 
    KeplerInverse, LiouvilleNumber, MahlerNumber, 
    ArcExpIntegralE, ArcExpIntegralEi, ArcLogIntegral, 
    ArcErf, ArcFresnelC, ArcFresnelS, Clothoid, ArcClothoid, 
    SuperExp, SuperLog, PeriodicSuperLog, 
    PearceyIntegral, SwallowtailIntegral, Cuspoid4DIntegral, 
    EllipticUmbilicIntegral, HyperbolicUmbilicIntegral *)

(*:Summary:
  This package implements miscellaneous special functions.
*)

(******************************************************************************)




BeginPackage["SpecialFunctions`Miscellaneous`"]




SievertIntegral::usage =
"SievertIntegral[z,t] gives the Sievert's integral
 function. Integrate[Exp[-z Sec[phi]],{phi,0,t}]."

SievertIntegralPrime::usage =
"SievertIntegralPrime[z,t] gives the derivative of
 Sievert's integral function on z."

SinSievertIntegral::usage =
"SinSievertIntegral[z,t] gives the Sievert's integral
 function. Integrate[Sin[z Sec[phi]],{phi,0,t}]."

SinSievertIntegralPrime::usage =
"SinSievertIntegralPrime[z,t] gives the derivative of
 sine Sievert's integral function on z."

CosSievertIntegral::usage =
"CosSievertIntegral[z,t] gives the Sievert's integral
 function. Integrate[Cos[z Sec[phi]],{phi,0,t}]."

CosSievertIntegralPrime::usage =
"CosSievertIntegralPrime[z,t] gives the derivative of
 cosine Sievert's integral function on z."

AbramowitzF::usage =
"AbramowitzF[m,z] gives the Abramowitz function."

AbramowitzFExp::usage =
"AbramowitzFExp[m,z] gives the Abramowitz function
 argument of Exp[z]."

CoefficientAbramowitzA::usage =
"CoefficientAbramowitzA[k] gives the coefficient values
 of Abramowitz function. Where, k is non negative
 integer number."

CoefficientAbramowitzB::usage =
"CoefficientAbramowitzB[k] gives the coefficient values
 of Abramowitz function. Where, k is non negative
 integer number."

KeplerInverse::usage =
"KeplerInverse[v,z] gives the Kepler's inverse function.
 It's solutions of equation z=K-v Sin(K) (0<=v<1)."

LiouvilleNumber::usage =
"LiouvilleNumber[z,a] gives the Liouville number function.
 L(z,a)=Sum[z^n/(a^(n!)),{n,1,Infinity}]."

MahlerNumber::usage =
"MahlerNumber[z,a,b] gives the Mahler number function.
 M(z,a,b)=Sum[z^n/(a^(b^n)),{n,1,Infinity}]."

ArcExpIntegralE::usage =
"ArcExpIntegralE[n,z] gives the inverse function of
 exponential integral."

ArcExpIntegralEi::usage =
"ArcExpIntegralEi[z] gives the inverse function of
 exponential integral."

ArcLogIntegral::usage =
"ArcLogIntegral[z] gives the inverse function of
 logarithmic integral."

ArcErf::usage =
"ArcErf[z] gives the inverse function of
 Error function."

ArcFresnelC::usage =
"ArcFresnelC[z] gives the inverse function of
 Fresnel cosine function."

ArcFresnelS::usage =
"ArcFresnelS[z] gives the inverse function of
 Fresnel sine function."

Clothoid::usage =
"Clothoid[z] gives the function of Clothoid curve."

ArcClothoid::usage =
"ArcClothoid[z] gives the inverse Clothoid curve function."

SuperExp::usage =
"SuperExp[z] gives the super exponential(tetration)
 function."

SuperLog::usage =
"SuperLog[z] gives the super logarithmic(inverse super
 exponential)function."

PeriodicSuperLog::usage =
"PeriodicSuperLog[z] gives the super logarithmic
 (inverse super exponential)function use other branch cut."

PearceyIntegral::usage =
"PearceyIntegral[x,y] gives the Pearcey's integral functions.
 Integrate[Exp[I*(t^4+y*t^2+x*t)],{t,-Infinity,Infinity}]."

SwallowtailIntegral::usage =
"SwallowtailIntegral[x,y,z] gives the Swallowtail canonical integral
 functions. Integrate[Exp[I*(t^5+z*t^3+y*t^2+x*t)],{t,-Infinity,Infinity}]."

Cuspoid4DIntegral::usage =
"Cuspoid4DIntegral[x,y,z,w] gives the Canonical Integral functions of
 Cuspoid Catastrophe with Codimension-4.
 Integrate[Exp[I*(t^6+w*t^4+z*t^3+y*t^2+x*t)],{t,-Infinity,Infinity}]."

EllipticUmbilicIntegral::usage =
"EllipticUmbilicIntegral[x,y,z] gives the Canonical Integral functions of
 Elliptic umbilic Catastrophe."

HyperbolicUmbilicIntegral::usage =
"HyperbolicUmbilicIntegral[x,y,z] gives the Canonical Integral functions of
 Hyperbolic umbilic Catastrophe."



Begin["SpecialFunctions`Miscellaneous`Private`"]






Attributes[SievertIntegral]={Listable}

Attributes[SievertIntegralPrime]={Listable}

Attributes[SinSievertIntegral]={Listable}

Attributes[SinSievertIntegralPrime]={Listable}

Attributes[CosSievertIntegral]={Listable}

Attributes[CosSievertIntegralPrime]={Listable}

Attributes[AbramowitzF]={Listable}

Attributes[AbramowitzFExp]={Listable}

Attributes[CoefficientAbramowitzA]={Listable}

Attributes[CoefficientAbramowitzB]={Listable}

Attributes[KeplerInverse]={Listable}

Attributes[LiouvilleNumber]={Listable}

Attributes[MahlerNumber]={Listable}

Attributes[ArcExpIntegralE]={Listable}

Attributes[ArcExpIntegralEi]={Listable}

Attributes[ArcLogIntegral]={Listable}

Attributes[ArcErf]={Listable}

Attributes[ArcFresnelC]={Listable}

Attributes[ArcFresnelS]={Listable}

Attributes[Clothoid]={Listable}

Attributes[ArcClothoid]={Listable}

Attributes[SuperExp]={Listable}

Attributes[SuperLog]={Listable}

Attributes[PeriodicSuperLog]={Listable}

Attributes[PearceyIntegral]={Listable}

Attributes[SwallowtailIntegral]={Listable}

Attributes[Cuspoid4DIntegral]={Listable}

Attributes[EllipticUmbilicIntegral]={Listable}

Attributes[HyperbolicUmbilicIntegral]={Listable}



(******************************************************************************)
(*                            SievertIntegral[z,t]                            *)
(******************************************************************************)

(*----------------------------------*)
(*         Sievert integral         *)
(*        (general formulae)        *)
(*----------------------------------*)

SievertIntegral[z_,t_]:=t/;(N[z]===0. || N[t]===0.)

SievertIntegral[z_,t_]:=
  -SievertIntegral[z,-t]/;(Re[t]<0)

(*----------------------------------*)
(*         Sievert integral         *)
(*      (0<=t<=Pi/2 && t:real)      *)
(*----------------------------------*)

SievertIntegral[z_,t_]:=
  integralOfBesselK0[z]-sievIntegF2[z,t]/;
  (Precision[z]<Infinity && Precision[t]<Infinity && 
  N[z]=!=0. && 0.301<=N[Re[t]]<=N[Pi/2] && N[Im[t]]===0.)

SievertIntegral[z_,t_]:=
  Sum[smallTTerm[n,z]/(n!)*t^n,{n,1,35,2}]*Exp[-z]/;
  (Precision[z]<Infinity && Precision[t]<Infinity && 
  N[z]=!=0. && 0<N[Re[t]]<0.301 && N[Im[t]]===0.)

sievIntegF2[z_,t_]:=
  Module[{s=0,so=-1,k1=0,af,cf,c1=Cos[t]},af=1;cf=Cos[t];
    While[Chop[so-s,10^-8]=!=0,so=s;
        s+=(af*cf*ExpIntegralE[2 k1+2,z/c1]);
        af*=(1-1/(2 k1+2));cf*=c1^2;k1++];s]/;
      (N[t]=!=N[Pi/2])

sievIntegF2[z_,t_]:=0/;(N[t]===N[Pi/2])

integralOfBesselK0[z_]:=
  ibkzero1[z]-ibkzero2[z]-ibkzero3[z]/;(Abs[z]<=15)

ibkzero1[z_]:=
  Module[{s=0,so=-1,k1=0,g1,zf,z1=(z^2/4)},g1=1;zf=1;
    While[so!=s,so=s;
        s+=N[zf/(g1*(2 k1+1))];
        g1*=((k1+1)^2);zf*=z1;k1++];s]*z*
        (EulerGamma+Log[z/2])+Pi/2;

ibkzero2[z_]:=
  Module[{s=0,so=-1,k1=0,g1,zf,z1=(z^2/4)},g1=1;zf=1;
    While[so!=s,so=s;
        s+=N[zf/(g1*(2 k1+1)^2)];
        g1*=((k1+1)^2);zf*=z1;k1++];s]*z;

ibkzero3[z_]:=
  Module[{s=0,so=-1,k1=1,g1,g2,zf,z1=(z^2/4)},
       g1=1;g2=1;zf=(z^2/4);
    While[so!=s,so=s;
        s+=N[zf*g2/(g1*(2 k1+1))];
        g1*=((k1+1)^2);g2+=(1/(k1+1));zf*=z1;k1++];s]*z;

integralOfBesselK0[z_]:=
  Module[{s=0,so=-1,k1=0,g1,g2,g3,zf,z1=(-1/z)},
       g1=1;g2=1;g3=1;zf=1;
    While[k1<Floor[Abs[z]],so=s;
        s+=N[zf*g1*g3];
        g1*=((2 k1+1)/2);g2*=((2 k1+1)/(4*(k1+1)));
        g3+=g2;zf*=z1;k1++];s]*Sqrt[Pi/(2 z)]*Exp[-z]/;
      (Abs[z]>15)

smallTTerm[1,z_]:=1

smallTTerm[3,z_]:=-z

smallTTerm[5,z_]:=z*(-5+3 z)

smallTTerm[7,z_]:=-z*(61-75 z+15 z^2)

smallTTerm[9,z_]:=z*(-1385+2583 z-1050 z^2+105 z^3)

smallTTerm[11,z_]:=
  -z*(50521-126375 z+77805 z^2-15750 z^3+945 z^4)

smallTTerm[13,z_]:=
  z*(-2702765+8481363 z-7005900 z^2+2144835 z^3-
  259875 z^4+10395 z^5)

smallTTerm[15,z_]:=
  -z*(199360981-752517675 z+780374595 z^2-
  320420100 z^3+58648590 z^4-4729725 z^5+135135 z^6)

smallTTerm[17,z_]:=
  z*(-19391512145+85541159343 z-106785683550 z^2+
  55047557565 z^3-13506743250 z^4+1644052410 z^5-
  94594500 z^6+2027025 z^7)

smallTTerm[19,z_]:=
  -z*(2404879675441-12139562880975 z+
  17722669607385 z^2-11000163406050 z^3+
  3388034584935 z^4-552568366350 z^5+
  47953735830 z^6-2067565500 z^7+34459425 z^8)

smallTTerm[21,z_]:=
  z*(-370371188237525+2105377838324523 z-
  3519373699338000 z^2+2555305230122295 z^3-
  947622622708125 z^4+193959079959645 z^5-
  22544504482500 z^6+1465283669850 z^7-
  49104680625 z^8+654729075 z^9)

smallTTerm[23,z_]:=
  -z*(69348874393137901-438363068860288275 z+
  825619413772328175 z^2-686579626455861600 z^3+
  297869643330931980 z^4-73401429328902675 z^5+
  10704538275315885 z^6-931509244165500 z^7+
  47036391477075 z^8-1260353469375 z^9+
  13749310575 z^10)

smallTTerm[25,z_]:=
  z*(-15514534163557086905+107946205931546904903 z-
  226178780608064633250 z^2+211979635209352343025 z^3-
  105345583638443032500 z^4+30369088894764935700 z^5-
  5331416719361733000 z^6+581963209080127875 z^7-
  39331922738983875 z^8+1587389987608425 z^9-
  34785755754750 z^10+316234143225 z^11)

smallTTerm[27,z_]:=
  -z*(4087072509293123892361-31038728292678856051575 z+
  71613294945418164448965 z^2-
  74687485727703092994750 z^3+
  41837198408630191291725 z^4-
  13815921946380034981500 z^5+
  2837279755613766563100 z^6-372770311680802035000 z^7+
  31596056003243481750 z^8-1706303437191478125 z^9+
  56300745689062875 z^10-1027760965481250 z^11+
  7905853580625 z^12)

smallTTerm[29,z_]:=
  z*(-1252259641403629865468285+
  10307332452501169055268483 z-
  25966217134332577213593300 z^2+
  29827236517769064444643755 z^3-
  18595318149312270705714375 z^4+
  6922140399046817874104175 z^5-
  1628389946857138399935000 z^6+
  250249687441003971220500 z^7-
  25526433117403822353750 z^8+
  1728509316959623448250 z^9-
  76297685242238460000 z^10+2096570703923821125 z^11-
  32374470412659375 z^12+213458046676875 z^13)

smallTTerm[31,z_]:=
  -z*(441543893249023104553682821-
  3915437485313561543531722875 z+
  10693868524127619615959641755 z^2-
  13415541214542361374586333500 z^3+
  9213243893734849599042946170 z^4-
  3817322858300098668141245625 z^5+
  1012285415397866423728189275 z^6-
  178195197829281263856225000 z^7+
  21259299046945688511032250 z^8-
  1732194400150599916781250 z^9+
  95855827636669284434250 z^10-
  3522959037476628750000 z^11+
  81868560779533027500 z^12-1083299586885140625 z^13+
  6190283353629375 z^14)

smallTTerm[33,z_]:=
  z*(-177519391579539289436664789665+
  1687184460280725496112090023263 z-
  4965790729551791305136653602150 z^2+
  6755261893556887303818832168485 z^3-
  5067308067527114267122385871750 z^4+
  2312994576234040773155920508670 z^5-
  682720550040856095854182597500 z^6+
  135474128316236933725664287125 z^7-
  18512412019228862963814836250 z^8+
  1764031170872840783636472750 z^9-
  117446102802477749207407500 z^10+
  5410546358577766769112750 z^11-
  168136170765304003762500 z^12+
  3347482387442035342500 z^13-
  38379756792502125000 z^14+191898783962510625 z^15)

smallTTerm[35,z_]:=
  -z*(80723299235887898062168247453281-
  818602547489556491973706927094175 z+
  2582934439455298727362646275738545 z^2-
  3787209136641599123499074367725850 z^3+
  3081015178848846860183845332057315 z^4-
  1536245402909329720901941735937850 z^5+
  499574045083861097664519125130210 z^6-
  110341835922136925043408373402500 z^7+
  16996290071293362570871372617375 z^8-
  1854913541875765162956458043750 z^9+
  144414644301512034594377483250 z^10-
  8003357594546605802323312500 z^11+
  311704654469515806970721250 z^12-
  8297346486800789155237500 z^13+
  143109669532746073657500 z^14-
  1435402904039579475000 z^15+6332659870762850625 z^16)

(*----------------------------------*)
(*         Sievert integral         *)
(*  ((t<0 && t>Pi/2) || t:complex)  *)
(*----------------------------------*)

SievertIntegral[z_,t_]:=
  sievIntegEven[z,t]-sievIntegOdd[z,t]+
  t+z^2/2*Tan[t]-z*lnpmsin[t]/;
  (Precision[z]<Infinity && Precision[t]<Infinity && 
  ((0<N[Re[t]]<=N[Pi/2] && Abs[Im[t]]=!=0.) || N[Re[t]]>N[Pi/2]))

sievIntegEven[z_,t_]:=
  Module[{s=0,so=-1,k1=2,g1,g2,g3,zf,z1=z^2,sf,cf,c1=Cos[t]^2},
    g1=24;g2=2;g3=3;zf=z^4;sf=Sin[t];cf=Cos[t]^3;
    While[so!=s,so=s;
      s+=N[zf*g2*sf*siEvenSum[k1,t]/(g1*g3*cf)];
      g1*=((2 k1+1) (2 k1+2));g2*=(2 k1);g3*=(2 k1+1);
      zf*=z1;cf*=c1;k1++];s]

siEvenSum[k_,t_]:=
  Sum[((2 k-2 r-3)!!)/((2 k-2 r-2)!!)*Cos[t]^(2 r),
  {r,0,k-1,1}];

sievIntegOdd[z_,t_]:=
  Module[{s=0,so=-1,k1=1,g1,g2,g3,zf,z1=z^2,sf,cf,c1=Cos[t]^2},
    g1=6;g2=1;g3=2;zf=z^3;sf=Sin[t];cf=Cos[t]^2;
    While[so!=s,so=s;
      s+=N[zf*g2/(g1*g3)*(sf/cf*siOddSum[k1,t]+lnpmsin[t])];
      g1*=((2 k1+2) (2 k1+3));g2*=(2 k1+1);g3*=(2 k1+2);
      zf*=z1;cf*=c1;k1++];s];

siOddSum[k_,t_]:=
  Sum[((2 k-2 r-2)!!)/((2 k-2 r-1)!!)*Cos[t]^(2 r),
  {r,0,k-1,1}];

lnpmsin[t_]:=Log[(1+Sin[t])/(1-Sin[t])]/2/;
  (N[-Pi/2]<Re[t]<=N[Pi/2])

lnpmsin[t_]:=-lnpmsin[t+Pi]-Pi I/;
  (Re[t]<=N[-Pi/2] && Im[t]<0)

lnpmsin[t_]:=-lnpmsin[t-Pi]+Pi I/;
  (Re[t]>N[Pi/2] && Im[t]>=0)

lnpmsin[t_]:=-lnpmsin[t+Pi]+Pi I/;
  (Re[t]<=N[-Pi/2] && Im[t]>=0)

lnpmsin[t_]:=-lnpmsin[t-Pi]-Pi I/;
  (Re[t]>N[Pi/2] && Im[t]<0)



(******************************************************************************)
(*                         SievertIntegralPrime[z,t]                          *)
(******************************************************************************)

(*----------------------------------*)
(*        (general formulae)        *)
(*----------------------------------*)

SievertIntegralPrime[z_,t_]:=
  -Log[(1+Sin[t])/(1-Sin[t])]/2/;(N[z]===0.)

SievertIntegralPrime[z_,t_]:=
  -SievertIntegralPrime[z,-t]/;(Re[t]<0)

(*----------------------------------*)
(*      (0<=t<=Pi/2 && t:real)      *)
(*----------------------------------*)

SievertIntegralPrime[z_,t_]:=
  -BesselK[0,z]+sievIntegPrime[z,t]/;
  (Precision[z]<Infinity && Precision[t]<Infinity && 
  N[z]=!=0. && 0.301<=N[Re[t]]<=N[Pi/2] && N[Im[t]]===0.)

SievertIntegralPrime[z_,t_]:=
  Sum[smallTTermPrime[n,z]/(n!)*t^n,{n,1,35,2}]*Exp[-z]/;
  (Precision[z]<Infinity && Precision[t]<Infinity && 
  N[z]=!=0. && 0<=N[Re[t]]<0.301 && N[Im[t]]===0.)

sievIntegPrime[z_,t_]:=
  Module[{s=0,so=-1,k1=0,af,cf,c1=Cos[t]},af=1;cf=1;
    While[Chop[so-s,10^-8]=!=0,so=s;
        s+=(af*cf*ExpIntegralE[2 k1+1,z/c1]);
        af*=(1-1/(2 k1+2));cf*=c1^2;k1++];s]/;
      (N[t]=!=N[Pi/2])

sievIntegPrime[z_,t_]:=0/;(N[t]===N[Pi/2])

smallTTermPrime[1,z_]:=-1
        
smallTTermPrime[3,z_]:=-1+z
        
smallTTermPrime[5,z_]:=-5+11*z-3*z^2
        
smallTTermPrime[7,z_]:=-61+211*z-120*z^2+15*z^3
        
smallTTermPrime[9,z_]:=
  -1385+6551*z-5733*z^2+1470*z^3-105*z^4
        
smallTTermPrime[11,z_]:=
  -50521+303271*z-359790*z^2+140805*z^3-20475*z^4+
  945*z^5
        
smallTTermPrime[13,z_]:=
  -2702765+19665491*z-29499063*z^2+15585240*z^3-
  3444210*z^4+322245*z^5-10395*z^6
        
smallTTermPrime[15,z_]:=
  -199360981+1704396331*z-3093641460*z^2+2062054995*z^3-
  613663050*z^4+87026940*z^5-5675670*z^6+135135*z^7
        
smallTTermPrime[17,z_]:=
  -19391512145+190473830831*z-405898209993*z^2+
  326975913810*z^3-122581273815*z^4+23371057710*z^5-
  2306213910*z^6+110810700*z^7-2027025*z^8
        
smallTTermPrime[19,z_]:=
  -2404879675441+26684005437391*z-65307571703130*z^2+
  61723323231585*z^3-27940336330725*z^4+
  6703444783035*z^5-888244517160*z^6+64494259830*z^7-
  2377700325*z^8+34459425*z^9
        
smallTTermPrime[21,z_]:=
  -370371188237525+4581126864886571*z-
  12663498936338523*z^2+13740594619827180*z^3-
  7293418343662920*z^4+2111377102465995*z^5-
  351770611337145*z^6+34266773841300*z^7-
  1907225795475*z^8+55651971375*z^9-654729075*z^10
        
smallTTermPrime[23,z_]:=
  -69348874393137901+946075012113714451*z-
  2915221310177272800*z^2+3571937919595774575*z^3-
  2175927843110521500*z^4+738278219304348030*z^5-
  148333197256113870*z^6+18156612228639885*z^7-
  1354836767459175*z^8+59639926170825*z^9-
  1411595885700*z^10+13749310575*z^11
        
smallTTermPrime[25,z_]:=
  -15514534163557086905+231406946026650896711*z-
  786482547755740804653*z^2+1074097321445474005350*z^3-
  738707553401567505525*z^4+287560117007032646700*z^5-
  67689005930297066700*z^6+9987122392002756000*z^7-
  935950513730982750*z^8+55205822615068125*z^9-
  1970033300910675*z^10+38580565473450*z^11-
  316234143225*z^12
        
smallTTermPrime[27,z_]:=
  -4087072509293123892361+66164529094650835995511*z-
  245878613128933349398470*z^2+
  370363237856230536427965*z^3-
  283873477770854049453375*z^4+
  124732730086910401180725*z^5-
  33676880235676400923200*z^6+5819442249060182843100*z^7-
  657134815709993370750*z^8+48659090375158263000*z^9-
  2325611639771169750*z^10+68633877274837875*z^11-
  1130537062029375*z^12+7905853580625*z^13
        
smallTTermPrime[29,z_]:=
  -1252259641403629865468285+
  21866924546405967976005251*z-
  88205983855498900696048383*z^2+
  145275163205408834992168320*z^3-
  122803827264330417973215630*z^4+
  60128160543593177950339425*z^5-
  18320870027046786673649175*z^6+
  3630387446385170169699000*z^7-
  479987585497638372404250*z^8+
  42811526287000056836250*z^9-
  2567783854624246508250*z^10+
  101456533689324313500*z^11-2517438819288393000*z^12+
  35362883066135625*z^13-213458046676875*z^14
        
smallTTermPrime[31,z_]:=
  -441543893249023104553682821+
  8272418863876146191617128571*z-
  35997043057696420391410648140*z^2+
  64356033382297065114304975755*z^3-
  59481760683216609369801064350*z^4+
  32117181043535441607890419920*z^5-
  10903320766085163634238570550*z^6+
  2437846998032116534577989275*z^7-
  369528889251792460455515250*z^8+
  38581243048451687678844750*z^9-
  2786608504153962045558000*z^10+
  138131336086388829434250*z^11-
  4587250327610558107500*z^12+97034754995924996250*z^13-
  1176153837189581250*z^14+6190283353629375*z^15
        
smallTTermPrime[33,z_]:=
  -177519391579539289436664789665+
  3551888312140990281660844836191*z-
  16584556648936099411522050829713*z^2+
  31986838303779340520411982276090*z^3-
  32091802231192458639430761527235*z^4+
  18945275524931358906057908923770*z^5-
  7092038426520033444135198691170*z^6+
  1766513576570751565659496894500*z^7-
  302085836489296700399997813375*z^8+
  36152723727957270800179563750*z^9-
  3055938301700096024917955250*z^10+
  182372659105410950436760500*z^11-
  7596316578526718818025250*z^12+
  215000924189492498557500*z^13-
  3923178739329567217500*z^14+
  41450137335902295000*z^15-191898783962510625*z^16
        
smallTTermPrime[35,z_]:=
  -80723299235887898062168247453281+
  1717928394215000882009582101641631*z-
  8567405865855452674061645754309810*z^2+
  17731770986021695221358943746641945*z^3-
  19192285030885833424418301028012425*z^4+
  12298487596304825185595495747684415*z^5-
  5033263718496357404553575611849320*z^6+
  1382308732460956498011786112350210*z^7-
  263308446563777188181250726958875*z^8+
  35545425490051014200435953054875*z^9-
  3443474629192397543494610359500*z^10+
  240454935436071304222257233250*z^11-
  12055518102650311292942688750*z^12+
  427867505284726855144046250*z^13-
  10443991529791980260100000*z^14+
  166076115997379345257500*z^15-
  1543058121842547935625*z^16+6332659870762850625*z^17



(******************************************************************************)
(*              SinSievertIntegral[z,t], CosSievertIntegral[z,t]              *)
(******************************************************************************)

SinSievertIntegral[z_,t_]:=
  (SievertIntegral[-I z,t]-SievertIntegral[I z,t])/(2 I)

CosSievertIntegral[z_,t_]:=
  (SievertIntegral[I z,t]+SievertIntegral[-I z,t])/2



(******************************************************************************)
(*         SinSievertIntegralPrime[z,t], CosSievertIntegralPrime[z,t]         *)
(******************************************************************************)

SinSievertIntegralPrime[z_,t_]:=
  (SievertIntegralPrime[-I z,t]+
  SievertIntegralPrime[I z,t])/(-2)

CosSievertIntegralPrime[z_,t_]:=
  (SievertIntegralPrime[-I z,t]-
  SievertIntegralPrime[I z,t])/(2 I)



(******************************************************************************)
(*                              AbramowitzF[m,z]                              *)
(******************************************************************************)

(*----------------------------------*)
(*            (general)             *)
(*----------------------------------*)

nearPosiIntQ[v_]:=
  Abs[v-Round[Re[v]]]<10^(-12) && Round[Re[v]]>=0

nearPosiIntQ[v_,n_]:=
  Abs[v-Round[Re[v]]]<10^(-12) && Round[Re[v]]===n

nearIntQ[v_]:=
  Abs[v-Round[Re[v]]]<10^(-12)

(*----------------------------------*)
(*          (m : Integer)           *)
(*----------------------------------*)

CoefficientAbramowitzA[0]:=0

CoefficientAbramowitzA[1]:=0

CoefficientAbramowitzA[2]:=-1

CoefficientAbramowitzB[0]:=1

CoefficientAbramowitzB[1]:=-Sqrt[Pi]

CoefficientAbramowitzB[2]:=3*(1-EulerGamma)/2

CoefficientAbramowitzA[k_Integer]:=
  -2*CoefficientAbramowitzA[k-2]/(k (k-1) (k-2))/;(k>2)

CoefficientAbramowitzB[k_Integer]:=
  -(2*CoefficientAbramowitzB[k-2]+(3 k^2-6 k+2)*
  CoefficientAbramowitzA[k])/(k (k-1) (k-2))/;(k>2)

coeffAbramowitzA[k_]:=
  coeffAbramowitzA[k]=CoefficientAbramowitzA[k]

coeffAbramowitzB[k_]:=
  coeffAbramowitzB[k]=CoefficientAbramowitzB[k]

AbramowitzF[0,0]:=Sqrt[Pi]/2

AbramowitzF[m_?EvenQ,0]:=
  Sqrt[Pi]*(m-1)!!/(2^(m/2+1))/;(m>=2)

AbramowitzF[m_?OddQ,0]:=
  (m-1)!!/(2^((m+1)/2))/;(m>=1)

AbramowitzF[m_,z_]:=
  Module[{s=0,so=-1,k1=1,zf,z1=z},zf=1;
   While[so!=s,so=s;
     s+=N[-zf*(coeffAbramowitzA[k1]+
          k1*(coeffAbramowitzA[k1]*Log[z]+
          coeffAbramowitzB[k1]))];
         zf*=z1;k1++];s]/2/;
       (nearPosiIntQ[m,0] && Precision[z]<Infinity && Abs[z]<=30)

AbramowitzF[m_,z_]:=
  Module[{s=0,so=-1,k1=0,zf,z1=z},zf=1;
   While[so!=s,so=s;
     s+=N[zf*(coeffAbramowitzA[k1]*Log[z]+
          coeffAbramowitzB[k1])];
         zf*=z1;k1++];s]/2/;
      (nearPosiIntQ[m,1] && Precision[z]<Infinity && Abs[z]<=30)

AbramowitzF[m_,z_]:=
  Module[{s=0,so=-1,k1=1,zf,z1=z},zf=1;
   While[so!=s,so=s;
     s+=N[zf*((2 k1-2)*coeffAbramowitzA[k1]+
          k1*(k1-2)*(coeffAbramowitzA[k1]*Log[z]+
          coeffAbramowitzB[k1]))];
          zf*=z1;k1++];s]/4/;
       (nearPosiIntQ[m,2] && Precision[z]<Infinity && Abs[z]<=30)

AbramowitzF[m_,z_]:=
  Module[{mr},mr=Round[m];
  (mr-1)/2*AbramowitzF[mr-2,z]+z/2*AbramowitzF[mr-3,z]]/;
  (nearPosiIntQ[m] && Re[m]>2 && Precision[z]<Infinity && Abs[z]<=30)

AbramowitzF[m_,z_]:=
  Module[{mr},mr=Round[m];
  2/z*AbramowitzF[mr+3,z]-(mr+2)/z*AbramowitzF[mr+1,z]]/;
  (nearIntQ[m] && Re[m]<0 && Precision[z]<Infinity)

asymCoeffAbramowitzA[k_Integer,m_Integer]:=
  asymCoeffAbramowitzA[k,m]=N[asymCoeffAA[k,m]]

asymCoeffAA[0,m_]:=1

asymCoeffAA[1,m_]:=(3 m^2+3 m-1)/12

asymCoeffAA[k_,m_]:=
  ((-1+12 k-12 k^2+3 m+3 m^2)*asymCoeffAA[k-1,m]+
  ((-1+2 k-m) (4-2 k+m) (-1+2 k+2 m))/2*
  asymCoeffAA[k-2,m])/(12 k)/;(k>=2)

asymAFTermV[z_]:=3*(2^(-2/3))*z^(2/3)

asymAFVPower[m_,z_]:=3^(m/2)*(2^(-m/3))*z^(m/3)

AbramowitzF[m_,z_]:=
  Module[{s=0,so=-1,k1=0,mr,zf,z1=asymAFTermV[z]},mr=Round[m];zf=1;
    While[Chop[so-s,10^-8]=!=0,so=s;
      s+=N[asymCoeffAbramowitzA[k1,mr]/zf];
      zf*=z1;k1++];s]*Sqrt[Pi/3]*3^(-m/2)*
      asymAFVPower[m,z]*Exp[-asymAFTermV[z]]/;
      (nearPosiIntQ[m] && Precision[z]<Infinity && Abs[z]>30)

(*----------------------------------*)
(*        (v : Not Integer)         *)
(*----------------------------------*)

AbramowitzF[v_,z_]:=
  1/2*(Gamma[(1+v)/2] HypergeometricPFQ[{},{1/2,1/2-v/2},-(z^2)/4]+
  z*(-Gamma[v/2] HypergeometricPFQ[{},{3/2,1-v/2},-(z^2)/4]+
  2*z^v*Gamma[-1-v] HypergeometricPFQ[{},{1+v/2,3/2+v/2},-(z^2)/4]))/;
  (nearIntQ[v]===False && Precision[z]<Infinity)



(******************************************************************************)
(*                            AbramowitzFExp[m,z]                             *)
(******************************************************************************)

(*----------------------------------*)
(*          (m : Integer)           *)
(*----------------------------------*)

asymAFExpIf[z_]:=If[(Abs[z-7]<=4 && Re[z]<6.5),1,0];

AbramowitzFExp[m_,z_]:=
  Module[{s=0,so=-1,k1=1,zf,z1=Exp[z]},zf=1;
   While[so!=s,so=s;
     s+=N[-zf*(coeffAbramowitzA[k1]+
          k1*(coeffAbramowitzA[k1]*z+
          coeffAbramowitzB[k1]))];
         zf*=z1;k1++];s]/2/;
         (nearPosiIntQ[m,0] && Precision[z]<Infinity && asymAFExpIf[z]===0)

AbramowitzFExp[m_,z_]:=
  Module[{s=0,so=-1,k1=0,zf,z1=Exp[z]},zf=1;
   While[so!=s,so=s;
     s+=N[zf*(coeffAbramowitzA[k1]*z+
          coeffAbramowitzB[k1])];
         zf*=z1;k1++];s]/2/;
         (nearPosiIntQ[m,1] && Precision[z]<Infinity && asymAFExpIf[z]===0)

AbramowitzFExp[m_,z_]:=
  Module[{s=0,so=-1,k1=1,zf,z1=Exp[z]},zf=1;
   While[so!=s,so=s;
     s+=N[zf*((2 k1-2)*coeffAbramowitzA[k1]+
          k1*(k1-2)*(coeffAbramowitzA[k1]*z+
          coeffAbramowitzB[k1]))];
          zf*=z1;k1++];s]/4/;
         (nearPosiIntQ[m,2] && Precision[z]<Infinity && asymAFExpIf[z]===0)

AbramowitzFExp[m_,z_]:=
  Module[{mr},mr=Round[m];
  (mr-1)/2*AbramowitzFExp[mr-2,z]+Exp[z]/2*AbramowitzFExp[mr-3,z]]/;
  (nearPosiIntQ[m] && Re[m]>2 && Precision[z]<Infinity && asymAFExpIf[z]===0)

AbramowitzFExp[m_,z_]:=
  Module[{mr},mr=Round[m];
  2*Exp[-z]*AbramowitzFExp[mr+3,z]-(mr+2)*Exp[-z]*AbramowitzFExp[mr+1,z]]/;
  (nearIntQ[m] && Re[m]<0 && Precision[z]<Infinity)

asymAFExpTermV[z_]:=3*(2^(-2/3))*Exp[2/3*z]

asymAFExpVPower[m_,z_]:=3^(m/2)*(2^(-m/3))*Exp[m/3*z]

AbramowitzFExp[m_,z_]:=
  Module[{s=0,so=-1,k1=0,mr,zf,z1=asymAFExpTermV[z]},mr=Round[m];zf=1;
    While[Chop[so-s,10^-8]=!=0,so=s;
    s+=N[asymCoeffAbramowitzA[k1,mr]/zf];
    zf*=z1;k1++];s]*Sqrt[Pi/3]*3^(-m/2)*
    asymAFExpVPower[m,z]*Exp[-asymAFExpTermV[z]]/;
   (nearPosiIntQ[m] && Precision[z]<Infinity && asymAFExpIf[z]===1)

(*----------------------------------*)
(*        (v : Not Integer)         *)
(*----------------------------------*)

setPrecAFE[z_,a_]:=
  If[(Abs[Arg[z]]<N[Pi/5] && Re[z]>3) || Re[z]>5.5,
     SetPrecision[a*10^100,100]/(10^100),a];

AbramowitzFExp[v_,z_]:=
  Module[{w,x},w=setPrecAFE[z,v];x=setPrecAFE[z,z];
  1/2*(Gamma[(1+w)/2]*HypergeometricPFQ[{},{1/2,1/2-w/2},-Exp[2 x]/4]+
  Exp[x]*(-Gamma[w/2]*HypergeometricPFQ[{},{3/2,1-w/2},-Exp[2 x]/4]+
  2*Exp[w x]*Gamma[-1-w]*HypergeometricPFQ[{},{1+w/2,3/2+w/2},-Exp[2 x]/4]))]/;
  (nearIntQ[v]===False && Precision[z]<Infinity)



(******************************************************************************)
(*                             KeplerInverse[v,z]                             *)
(******************************************************************************)

KeplerInverse[v_,0]:=0

KeplerInverse[v_,z_]:=z/;(N[v]===0.)

KeplerInverse[v_,z_]:=
  (FindRoot[ans-v Sin[ans]-z==0,
  {ans,Pi/4*z}])[[1,2]]/;
  (Precision[z]<Infinity && N[-Pi]<=Re[z]<N[Pi] && 
  -1<Re[v]<0)

KeplerInverse[v_,z_]:=
  (FindRoot[ans-v Sin[ans]-z==0,
  {ans,z/(1-v)}])[[1,2]]/;
  (Precision[z]<Infinity && N[-Pi]<=Re[z]<N[Pi] && 
  Re[v]<=-1)

KeplerInverse[v_,z_]:=
  KeplerInverse[v,z-2 N[Pi]]+2 Pi/;
  (Re[z]>=N[Pi] && Re[v]<0)

KeplerInverse[v_,z_]:=
  KeplerInverse[v,z+2 N[Pi]]-2 Pi/;
  (Re[z]<N[-Pi] && Re[v]<0)

KeplerInverse[v_,z_]:=
  KeplerInverse[-v,z-N[Pi]]+Pi/;(Re[v]>0)



(******************************************************************************)
(*                            LiouvilleNumber[z,a]                            *)
(******************************************************************************)

LiouvilleNumber[z_,a_]:=
  Module[{s=0,so=1,k1=1,zf=z,gf=1,af=a},
	While[so!=s,so=s; s+=zf/af;
	zf*=z; af*=(a^(gf k1)); gf*=(k1+1); k1++];s]/;
	(Precision[z]<Infinity && Abs[a]>1)

LiouvilleNumber[z_,a_]:=
  z/(1-z)/;(N[Abs[a]]===1.)

LiouvilleNumber[a_]:=
  LiouvilleNumber[1,a]/;(Abs[a]>1)



(******************************************************************************)
(*                            MahlerNumber[z,a,b]                             *)
(******************************************************************************)

MahlerNumber[z_,a_,b_]:=
  Module[{s=0,so=1,zf=z,af=(a^b),bf=b},
	While[so!=s,so=s; s+=zf/af;
	zf*=z; af*=(a^(bf (b-1))); bf*=b];s]/;
	(Precision[z]<Infinity && Abs[a]>1 && Abs[b]>1)

MahlerNumber[z_,a_,b_]:=
  z/(1-z)/a/;(N[Abs[b]]===1. && N[a]=!=0.)

MahlerNumber[a_,b_]:=
  MahlerNumber[1,a,b]/;(Abs[a]>1 && Abs[b]>1)



(******************************************************************************)
(*                            ArcExpIntegralE[n,z]                            *)
(******************************************************************************)

ArcExpIntegralE[n_,z_]:=
  (FindRoot[ExpIntegralE[n,ans]-z==0,{ans,initArcE[z]}])[[1,2]]/;
  (Precision[z]<Infinity && -Pi<Im[z]<=Pi)

ArcExpIntegralE[n_,z_]:=ArcExpIntegralE[n,z-2 Pi I]/;(Im[z]>Pi)

ArcExpIntegralE[n_,z_]:=ArcExpIntegralE[n,z+2 Pi I]/;(Im[z]<=-Pi)

initArcE[z_]:=ArcCsch[z];



(******************************************************************************)
(*                            ArcExpIntegralEi[z]                             *)
(******************************************************************************)

ArcExpIntegralEi[z_Real]:=
  Abs[(FindRoot[ExpIntegralEi[ans]-z==0,{ans,initArcEi[z]}])[[1,2]]]/;
  (Precision[z]<Infinity)

ArcExpIntegralEi[z_Complex]:=
  (FindRoot[ExpIntegralEi[ans]-z==0,{ans,initArcEi[z]}])[[1,2]]/;
  (Precision[z]<Infinity && -Pi<Im[z]<=Pi)

ArcExpIntegralEi[z_]:=ArcExpIntegralEi[z-2 Pi I]/;(Im[z]>Pi)

ArcExpIntegralEi[z_]:=ArcExpIntegralEi[z+2 Pi I]/;(Im[z]<=-Pi)

initArcEi[z_]:=
  If[Re[z]>1.354577831141296,ProductLog[z],
  ArcCsch[2.7091556622825914-z]];



(******************************************************************************)
(*                             ArcLogIntegral[z]                              *)
(******************************************************************************)

ArcLogIntegral[z_]:=Exp[ArcExpIntegralEi[z]];



(******************************************************************************)
(*                                 ArcErf[z]                                  *)
(******************************************************************************)

ArcErf[0]:=0;

ArcErf[z_]:=
  (FindRoot[Erf[ans]-z==0,{ans,initArcErf[z]}])[[1,2]]/;
  (Precision[z]<Infinity)

initArcErf[z_]:=
  If[Abs[z-1.02]<0.08,arcErfPlus1[z],
    If[Abs[z+1.02]<0.08,arcErfMinus1[z],ArcTanh[z]]];

arcErfPlus1[z_]:=
  Sqrt[Log[2/Pi]-2*Log[-z+1]-Log[Log[2/Pi]-2*Log[-z+1]]]/
  Sqrt[2];

arcErfMinus1[z_]:=
  -Sqrt[Log[2/Pi]-2*Log[z+1]-Log[Log[2/Pi]-2*Log[z+1]]]/
  Sqrt[2];



(******************************************************************************)
(*                       ArcFresnelC[z], ArcFresnelS[z]                       *)
(******************************************************************************)

ArcFresnelC[0]:=0;

ArcFresnelC[z_]:=
  FindRoot[FresnelC[ans]-z==0,{ans,initFuncAFC[z]}][[1,2]]/;
  (Precision[z]<Infinity)

initFuncAFC[z_]:=
  If[Abs[z]>FresnelC[1.],
    FresnelC[1.]*ArcSin[(z/FresnelC[1.])^2]/z,
    2/Pi*ArcSin[z/FresnelC[1.]]];

ArcFresnelS[0]:=0;

ArcFresnelS[z_]:=
  FindRoot[FresnelS[ans]-z==0,
  {ans,(4*Sqrt[2]/Pi*ArcSin[z/FresnelS[Sqrt[2.]]])^(1/3)}][[1,2]]/;
  (Precision[z]<Infinity)

Clothoid[z_]:=
  FresnelS[ArcFresnelC[z]]/;(Precision[z]<Infinity)

ArcClothoid[z_]:=
  FresnelC[ArcFresnelS[z]]/;(Precision[z]<Infinity)



(******************************************************************************)
(*                        SuperExp[a,z], SuperLog[a,z]                        *)
(******************************************************************************)

slSingularRe=
  0.31813150520476413531265425158766451720351761387139986692237860622941;

slSingularIm=
  1.33723570143068940890116214319371061253950213846051241887631278191435;

slSingularAbs=Abs[slSingularRe+slSingularIm*I];

SuperLog[z_]:=
  superLg[superLogList[150],z]/;
  (Precision[z]<Infinity && Abs[z]<slSingularAbs)

SuperLog[z_]:=
  SuperLog[Exp[z]]-1/;
  (Abs[z]>=slSingularAbs && Re[z]<=slSingularRe)

SuperLog[z_]:=
  SuperLog[Log[z]]+1/;
  (Abs[z]>=slSingularAbs && Re[z]>slSingularRe)

superLogList[n_Integer]:=
  superLogList[n]=
  LinearSolve[Table[k^j/k!-If[j===k,1,0],
  {j,0,n-1},{k,1,n}],Table[If[k===1,1,0],{k,1,n}]];

superLg[slList_,z_?NumericQ]:=
  Sum[slList[[k]]*z^k/k!,{k,1,Length[slList]}]-1;

SuperExp[z_]:=
  If[sexpRegionQ[z],asymSuperExp[z],smalSuperExp[z]]/;
  (Precision[z]<Infinity)

asymSuperExp[z_]:=
  If[Im[z]<0,Conjugate[asymSuperExp[Conjugate[z]]],
    If[0<=Im[z]<(4*Exp[Re[z]*Log[1.4758]/8]),
       Exp[asymSuperExp[z-1]],asymSE[z]]];

asymSE[z_]:=
  Module[{s,a2,a3,a4,a5,et},
    s=slSingularRe+slSingularIm*I;
    a2=N[1/2/(s-1),60];
    a3=(a2+1/6)/(s^2-1);
    a4=(a2/2+a2^2/2+a3+1/24)/(s^3-1);
    a5=(a2^2/2+a2/6+a2*a3+a3/2+a4+1/120)/(s^4-1);
    et=Exp[s*z+(1.0779614375278-0.9465409639479I)];
    s+et+a2*et^2+a3*et^3+a4*et^4+a5*et^5+
      (0.1223-0.02370I)*et*Exp[2*Pi*I*z]];

smalSuperExp[z_]:=
  FindRoot[SuperLog[y]==z,{y,fitFunctSE[z]}][[1,2]]/;
  (-1<Re[z]<=0)

smalSuperExp[z_]:=
  Log[smalSuperExp[z+1]]/;
  (Re[z]<=-1)

smalSuperExp[z_]:=
  Module[{se},se=smalSuperExp[z-1];
  If[N[-100*Log[10]]<Abs[se]<N[100*Log[10]],Exp[se],se]]/;
  (Re[z]>0)

fitFunctSE[z_]:=
  If[Abs[Im[z]]>1.3,
  asymSuperExp[z],
  0.6*(Log[z+2]+(z+1)*(1+z/2*(E-2+Log[4/3])*
    Exp[(z-1)*(Exp[Exp[z-2.51]]-0.6+0.08*(z+1))]-Log[2]))+
    0.4*Log[Log[z+3]+(z+2)*(1+(z+1)/2*(E-2+Log[4/3])*
    Exp[z*(Exp[Exp[z-1.51]]-0.6+0.08*z)]-Log[2])]];

sexpRegionQ[z_]:=
    If[Abs[Im[z]]>1.8*Exp[Re[z]*Log[3/2]/8],True,False];

PeriodicSuperLog[z_]:=
  superLg[superLogList[150],z]/;
  (Precision[z]<Infinity && Abs[z]<slSingularAbs)

PeriodicSuperLog[z_]:=
  PeriodicSuperLog[Exp[z]]-1/;
  (Abs[z]>=slSingularAbs && periodSLRegionQ[z]===False && 
   Abs[Im[z]]<=N[Pi])

PeriodicSuperLog[z_]:=
  PeriodicSuperLog[Log[z]]+1/;
  (Abs[z]>=slSingularAbs && periodSLRegionQ[z]===True)

PeriodicSuperLog[z_]:=
  PeriodicSuperLog[z-Sign[Im[z]]*2 Pi I]/;
  (Abs[Im[z]]>N[Pi])

periodSLRegionQ[z_]:=
  If[Abs[Im[z]]<=slSingularIm && Re[z]>=slSingularRe,True,False];



(******************************************************************************)
(*                            PearceyIntegral[x,y]                            *)
(******************************************************************************)

PearceyIntegral[x_,y_]:=
  Pi/2*Sqrt[Abs[y]/2]*Exp[-I/8*y^2]*
  (Exp[Pi*I/8]*BesselJ[-1/4,y^2/8]-Sign[y]*Exp[-Pi*I/8]*BesselJ[1/4,y^2/8])/;
  (N[x]===0.)

PearceyIntegral[x_,y_]:=
  Module[{so=0,s,xp,yp,prec=400,n=1,nMax=600,a0,a1,aN,aN1},
    xp=SetPrecision[x,prec];
    yp=SetPrecision[y,prec];
    s=Exp[Pi*I/8]*Gamma[1/4];
    a0=1;
    a1=I*xp;
   While[(Abs[N[so-s]]>10^-10 || n<nMax),so=s;
     aN=I/(2 n)*(xp*a1+2*yp*a0);
     aN1=I/(2 n+1)*(xp*aN+2*yp*a1);
     s+=(Exp[(2 n+1)/8*I*Pi]*Gamma[(2 n+1)/4]*aN);
	 a0=aN;a1=aN1;n++];
   N[s/2]]/;
  (N[x]=!=0.)



(******************************************************************************)
(*                         SwallowtailIntegral[x,y,z]                         *)
(******************************************************************************)

SwallowtailIntegral[x_,y_,z_]:=
  Module[{xp,yp,zp,prec=500,cosList,gamList,nMax,ans},
    xp=SetPrecision[x,prec];
    yp=SetPrecision[y,prec];
    zp=SetPrecision[z,prec];
    nMax=nMaxFuncSwallow[z];
    cosList=Table[I^n*Cos[(4 n-1)/10*Pi],{n,0,nMax}];
    gamList=Table[Gamma[(n+1)/5],{n,0,nMax}];
    ans=Apply[Plus,stiAList[{xp,yp,zp},nMax]*cosList*gamList];
   Clear[xp,yp,zp,cosList,gamList];
   N[2/5*ans]];

stiAList[{x_,y_,z_},nMax_]:=
  Module[{listA,a0,a1,a2,a3,n=3},
    a0=1;
    a1=I*x;
    a2=I*y-1/2*x^2;
    listA={a0,a1,a2};
   While[n<=nMax,
     a3=I/n*(x*a2+2*y*a1+3*z*a0);
     listA=Join[listA,{a3}];
     a0=a1;a1=a2;a2=a3;n++];
    listA];

nMaxFuncSwallow[z_]:=
  If[Re[z]>-0.1,nmfPlus[z],If[Re[z]<-1.2,nmfMinus[z],713]];

nmfPlus[z_]:=
  Ceiling[722.7923947699867+100.43430367852055*z+
  17.894257204584363*z^2+0.0027602926812687755*Exp[z]];

nmfMinus[z_]:=
  31+Ceiling[488.3836935401392-149.23059331232858*z+
  9.867435781204573*z^2+0.021650287607572254*Exp[-z]];



(******************************************************************************)
(*                         Cuspoid4DIntegral[x,y,z,w]                         *)
(******************************************************************************)

Cuspoid4DIntegral[x_,y_,z_,w_]:=
  Module[{so=0,s,xp,yp,zp,wp,prec=300,n=2,nMax=500,a0,a1,a2,a3,aN,aN1},
    xp=SetPrecision[x,prec];
    yp=SetPrecision[y,prec];
    zp=SetPrecision[z,prec];
    wp=SetPrecision[w,prec];
    s=Exp[1/12*I*Pi]*Gamma[1/6]+Exp[1/4*I*Pi]*Gamma[1/2]*(I*yp-1/2*xp^2);
    a0=1;
    a1=I*xp;
    a2=I*yp-1/2*xp^2;
    a3=I*zp-xp*yp-I/6*xp^3;
   While[(Abs[N[so-s]]>10^-10 || n<nMax),so=s;
     aN=I/(2 n)*(xp*a3+2*yp*a2+3*zp*a1+4*wp*a0);
     aN1=I/(2 n+1)*(xp*aN+2*yp*a3+3*zp*a2+4*wp*a1);
     s+=(Exp[(2 n+1)/12*I*Pi]*Gamma[(2 n+1)/6]*aN);
	 a0=a2;a1=a3;a2=aN;a3=aN1;n++];
   N[s/3]];



(******************************************************************************)
(*                               cdList[t,nMax]                               *)
(******************************************************************************)

cdList[t_,nMax_]:=
  Module[{c0,d0,c1,d1,lenC,lenD,listC,listD,n=3},
    c0={1};
    d0={0,1};
    listC={1,0,t,1};
    listD={0,1,0,t};
   While[n<nMax,
     lenC=Length[c0];
     lenD=Length[d0];
     c1=PadRight[Rest[c0]*Range[lenC-1],lenD+1]+Join[{0},d0];
     d1=c0+PadRight[Rest[d0]*Range[lenD-1],lenC];
     listC=Join[listC,{FromDigits[Reverse[c1],t]}];
     listD=Join[listD,{FromDigits[Reverse[d1],t]}];
     c0=c1;
     d0=d1;n++];
   {listC,listD}];



(******************************************************************************)
(*                       EllipticUmbilicIntegral[x,y,z]                       *)
(******************************************************************************)

EllipticUmbilicIntegral[x_,y_,z_]:=
  Module[{prec=50,zp,w1,w2,nMax,zList,c1,c2,d1,d2,ans},
    zp=SetPrecision[z,prec];
    w1=SetPrecision[(x+I*y)/(12^(1/3)),prec];
    w2=SetPrecision[(x-I*y)/(12^(1/3)),prec];
    nMax=nMaxFuncEllipticUmb[z];
    zList=Table[(-I*(2/3)^(2/3)*zp)^n/(n!),{n,0,nMax}];
    {c1,d1}=cdList[w1,nMax];
    {c2,d2}=cdList[w2,nMax];
    ans=Re[(c1*AiryAi[w1]+d1*AiryAiPrime[w1])*
           (c2*AiryBi[w2]+d2*AiryBiPrime[w2])].zList;
   Clear[zp,w1,w2,zList,c1,c2,d1,d2];
   N[2*Pi^2*(2/3)^(2/3)*ans]];

nMaxFuncEllipticUmb[z_]:=
  Ceiling[4.245343247230728+8.64930767360211*Abs[z]+
    4.9172836135405795*Abs[z]^2+0.7505547881254452*Abs[z]^3+
    0.01247479154704607*Exp[Abs[z]]+8.396742423817534*Exp[-Abs[z]]];



(******************************************************************************)
(*                      HyperbolicUmbilicIntegral[x,y,z]                      *)
(******************************************************************************)

HyperbolicUmbilicIntegral[x_,y_,z_]:=
  Module[{xc=x/(3^(1/3)),yc=y/(3^(1/3)),zc=z/(3^(1/3))},
   N[4*Pi^2*3^(-2/3)*huiFunc[xc,yc,zc]]];

huiFunc[x_,y_,z_]:=
  Module[{xp,yp,zp,prec=50,zList,facList,c1,c2,d1,d2,nMax=100,
          ans1,ans2,ans3,ans4},
    xp=SetPrecision[x,prec];
    yp=SetPrecision[y,prec];
    zp=SetPrecision[z,prec];
    zList=Table[(-I*3^(-1/3)*zp)^n,{n,2,nMax}];
    facList=Table[n!,{n,2,nMax}];
    {c1,d1}=Drop[cdList[xp,nMax],None,2];
    {c2,d2}=Drop[cdList[yp,nMax],None,2];
    ans1=Apply[Plus,c1*c2*zList/facList];
    ans2=Apply[Plus,c1*d2*zList/facList];
    ans3=Apply[Plus,d1*c2*zList/facList];
    ans4=Apply[Plus,d1*d2*zList/facList];
   Clear[zList,facList,c1,c2,d1,d2];
   AiryAi[xp]*AiryAi[yp]*(1+ans1)+AiryAi[xp]*AiryAiPrime[yp]*ans2+
   AiryAiPrime[xp]*AiryAi[yp]*ans3+
   AiryAiPrime[xp]*AiryAiPrime[yp]*((-I*3^(-1/3)*zp)+ans4)];



(******************************************************************************)

End[]

Protect[SievertIntegral, SievertIntegralPrime, SinSievertIntegral, 
    SinSievertIntegralPrime, CosSievertIntegral, 
    CosSievertIntegralPrime, AbramowitzF, AbramowitzFExp, 
    KeplerInverse, LiouvilleNumber, MahlerNumber, 
    ArcExpIntegralE, ArcExpIntegralEi, ArcLogIntegral, 
    ArcErf, ArcFresnelC, ArcFresnelS, Clothoid, ArcClothoid, 
    SuperExp, SuperLog, PeriodicSuperLog, 
    PearceyIntegral, SwallowtailIntegral, Cuspoid4DIntegral, 
    EllipticUmbilicIntegral, HyperbolicUmbilicIntegral]

EndPackage[]
